import string
import re
from dsympy import DSYMPY_DEBUG

import sympy


class SympyParser(object):
    """Abstract class for concrete parsers"""
    def __init__(self, expr):
        self.expr = expr

    def parse(self):
        raise NotImplementedError

class SympyExprParser(SympyParser):
    """"
    Parses an expression and tries to represent this as a Sympy formula

    It accepts Mathematica-like syntax, Sympy-like syntax or a mixture of both

    Examples
    ========
    >>> my_expr = '2 x + 7^x'
    >>> SympyParser(my_expr).parse()
    2*x+7**x
    """

    def extract_symbols(self):
        matches = re.findall(r'(\A|\W)([a-z]\_[0-9]+|[a-z]|[A-Z])(\Z|\W)', unicode(self.expr))
        return [m[1] for m in matches]

    def parse(self):
        expr = unicode(self.expr)
        if DSYMPY_DEBUG:
            print u"calling sympify with expression %s" % expr
        expr = string.replace(expr, '^', '**') # powers in python are represented by **
        expr = re.sub(r'(?P<s1>[a-z0-9])\s+(?P<s2>[a-z0-9])', r'\g<s1>*\g<s2>', expr) # 2 x -> 2*x
        return expr

class SympyMethodParser(SympyParser):
    
    def parse(self):
        return unicode(self.expr)